@@ -305,14 +305,16 @@ def test_subscription(worker1, worker2, collector, pub, topic):
     sub1.set(zmq.SUBSCRIBE, topic)
     # sync_pubsub will disturb the worker. so the worker should be stopped
     # during sync_pubsub works.
-    worker2.stop()
+    worker1.stop()
     sync_pubsub(pub, [sub1], topic)
-    worker2.start()
+    worker1.start()
     assert len(fanout.emit(topic, 'zeronimo')) == 1
     sub2.set(zmq.SUBSCRIBE, topic)
     # same reason
+    worker1.stop()
     worker2.stop()
     sync_pubsub(pub, [sub2], topic)
+    worker1.start()
     worker2.start()
     assert len(fanout.emit(topic, 'zeronimo')) == 2
 
@@ -624,19 +626,22 @@ def test_marshal_message(ctx, addr1, addr2):
 
 def test_exception_handler(worker, collector, push, capsys):
     customer = zeronimo.Customer(push, collector)
+    # without exception handler
+    with pytest.raises(ZeroDivisionError):
+        customer.emit('zero_div').get()
+    out, err = capsys.readouterr()
+    assert 'ZeroDivisionError:' in err
+    # with exception handler
     exceptions = []
     def exception_handler(exc_info):
         exceptions.append(exc_info[0])
-    customer.emit('zero_div').wait()
-    out, err = capsys.readouterr()
-    assert 'ZeroDivisionError' in err
-    assert len(exceptions) == 0
     worker.exception_handler = exception_handler
     with pytest.raises(ZeroDivisionError):
         customer.emit('zero_div').get()
     out, err = capsys.readouterr()
-    assert 'ZeroDivisionError' not in err
+    assert 'ZeroDivisionError:' not in err
     assert len(exceptions) == 1
+    assert exceptions[0] is ZeroDivisionError
 
 
 class ExampleException(BaseException):
